@@ -6,11 +6,12 @@
 #
 """ This module gathers list/line operations """
 
-from textops import TextOp
+from textops import TextOp, dformat
 import re
 import subprocess
 import sys
 import os
+import itertools
 
 class ListOpError(Exception):
     pass
@@ -32,6 +33,8 @@ class cat(TextOp):
         <generator object extend_type_gen at ...>
         >>> '/tmp/testfile.txt' | cat().tostr()
         'here is the file content'
+        >>> '/tmp/testfile.txt' >> cat()
+        ['here is the file content']
         >>> '/tmp/testfile.txt' | cat().upper().tostr()
         'HERE IS THE FILE CONTENT'
         >>> context = {'path':'/tmp/'}
@@ -103,6 +106,8 @@ class run(TextOp):
         f1
         f2
         f3
+        >>> print cmd >> run()
+        ['f1', 'f2', 'f3']
         >>> print ['ls', '/tmp/textops_tests_run'] | run().tostr()
         f1
         f2
@@ -130,7 +135,7 @@ class run(TextOp):
 class mrun(TextOp):
     r""" Run multiple commands from the input text and return execution output
 
-    | This works like textops.run_ except that each line of the input text will be used as a command.
+    | This works like :class:`textops.run` except that each line of the input text will be used as a command.
     | The input text must be a list of strings (list, generator, or newline separated), \
       not a list of lists. Commands will be executed inside a shell.
     | If a context dict is specified, commands are formatted with that context (str.format)
@@ -156,6 +161,8 @@ class mrun(TextOp):
         f1
         f2
         f3
+        >>> print cmds >> mrun()
+        ['f1', 'f2', 'f3']
         >>> cmds = ['ls {path}', 'echo "Cool !"']
         >>> print cmds | mrun({'path':'/tmp/textops_tests_run'}).tostr()
         f1
@@ -198,6 +205,8 @@ class grep(TextOp):
         <generator object extend_type_gen at ...>
         >>> input | grep('error').tolist()
         ['error1', 'error2']
+        >>> input >> grep('error')
+        ['error1', 'error2']
         >>> input | grep('ERROR').tolist()
         []
         >>> input | grep('error|warning').tolist()
@@ -245,7 +254,7 @@ class grep(TextOp):
 class grepi(grep):
     r"""grep case insensitive
 
-    This works like textops.grep_, except it is case insensitive.
+    This works like :class:`textops.grep`, except it is case insensitive.
 
     Args:
         pattern (str): a regular expression string (case insensitive)
@@ -258,13 +267,15 @@ class grepi(grep):
         >>> input = 'error1\nerror2\nwarning1\ninfo1\nwarning2\ninfo2'
         >>> input | grepi('ERROR').tolist()
         ['error1', 'error2']
+        >>> input >> grepi('ERROR')
+        ['error1', 'error2']
     """
     flags = re.IGNORECASE
 
 class grepv(grep):
     r"""grep with inverted matching
 
-    This works like textops.grep_, except it returns lines that does NOT match the specified pattern.
+    This works like :class:`textops.grep`, except it returns lines that does NOT match the specified pattern.
 
     Args:
         pattern (str): a regular expression string
@@ -277,6 +288,8 @@ class grepv(grep):
         >>> input = 'error1\nerror2\nwarning1\ninfo1\nwarning2\ninfo2'
         >>> input | grepv('error').tolist()
         ['warning1', 'info1', 'warning2', 'info2']
+        >>> input >> grepv('error')
+        ['warning1', 'info1', 'warning2', 'info2']
         >>> input | grepv('ERROR').tolist()
         ['error1', 'error2', 'warning1', 'info1', 'warning2', 'info2']
     """
@@ -285,7 +298,7 @@ class grepv(grep):
 class grepvi(grepv):
     r"""grep case insensitive with inverted matching
 
-    This works like textops.grepv_, except it is case insensitive.
+    This works like :class:`textops.grepv`, except it is case insensitive.
 
     Args:
         pattern (str): a regular expression string (case insensitive)
@@ -298,13 +311,15 @@ class grepvi(grepv):
         >>> input = 'error1\nerror2\nwarning1\ninfo1\nwarning2\ninfo2'
         >>> input | grepvi('ERROR').tolist()
         ['warning1', 'info1', 'warning2', 'info2']
+        >>> input >> grepvi('ERROR')
+        ['warning1', 'info1', 'warning2', 'info2']
     """
     flags = re.IGNORECASE
 
 class grepc(TextOp):
     r"""Count lines having a specified pattern
 
-    This works like textops.grep_ except that instead of filtering the input text,
+    This works like :class:`textops.grep` except that instead of filtering the input text,
     it counts lines matching the pattern.
 
     Args:
@@ -366,7 +381,7 @@ class grepc(TextOp):
 class grepci(grepc):
     r"""Count lines having a specified pattern (case insensitive)
 
-    This works like textops.grepc_ except that the pattern is case insensitive
+    This works like :class:`textops.grepc` except that the pattern is case insensitive
 
     Args:
         pattern (str): a regular expression string (case insensitive)
@@ -385,7 +400,7 @@ class grepci(grepc):
 class grepcv(grepc):
     r"""Count lines NOT having a specified pattern
 
-    This works like textops.grepc_ except that it counts line that does NOT match the pattern.
+    This works like :class:`textops.grepc` except that it counts line that does NOT match the pattern.
 
     Args:
         pattern (str): a regular expression string (case sensitive)
@@ -406,7 +421,7 @@ class grepcv(grepc):
 class grepcvi(grepcv):
     r"""Count lines NOT having a specified pattern (case insensitive)
 
-    This works like textops.grepcv_ except that the pattern is case insensitive
+    This works like :class:`textops.grepcv` except that the pattern is case insensitive
 
     Args:
         pattern (str): a regular expression string (case insensitive)
@@ -426,7 +441,7 @@ class haspattern(grepc):
     r"""Tests if the input text matches the specified pattern
 
     This reads the input text line by line (or item by item for lists and generators), cast into
-    a string before testing. like textops.grepc_ it accepts testing on a specific column
+    a string before testing. like :class:`textops.grepc` it accepts testing on a specific column
     for a list of lists or testing on a specific key for list of dicts.
     It stops reading the input text as soon as the pattern is found : it is useful for big input text.
 
@@ -449,7 +464,7 @@ class haspattern(grepc):
 class haspatterni(haspattern):
     r"""Tests if the input text matches the specified pattern
 
-    Works like textops.haspattern_ except that it is case insensitive.
+    Works like :class:`textops.haspattern` except that it is case insensitive.
 
     Args:
         pattern (str): a regular expression string (case insensitive)
@@ -487,6 +502,12 @@ class formatitems(TextOp):
     r"""Formats list of 2-sized tuples
 
     Useful to convert list of 2-sized tuples into a simple string
+    It converts the list of tuple into a list of strings by using the ``format_str``, then it
+    joins all the strings with ``join_str`` to get a unique simple string.
+
+    Args:
+        format_str(str): format string, default is '{0} : {1}\n'
+        join_str(str): string to join all strings into one unique string, default is ''
 
     Returns:
         str: formatted input
@@ -505,6 +526,12 @@ class formatlists(TextOp):
     r"""Formats list of lists
 
     Useful to convert list of lists into a simple string
+    It converts the list of lists into a list of strings by using the ``format_str``, then it
+    joins all the strings with ``join_str`` to get a unique simple string.
+
+    Args:
+        format_str(str): format string, default is '{0} : {1}\n'
+        join_str(str): string to join all strings into one unique string, default is ''
 
     Returns:
         str: formatted input
@@ -522,28 +549,35 @@ class formatlists(TextOp):
 class formatdicts(TextOp):
     r"""Formats list of dicts
 
-    Useful to convert list of dicts into a simple string
+    Useful to convert list of dicts into a simple string.
+    It converts the list of dicts into a list of strings by using the ``format_str``, then it
+    joins all the strings with ``join_str`` to get a unique simple string.
+
+    Args:
+        format_str(str): format string, default is '{key} : {val}\n'
+        join_str(str): string to join all strings into one unique string, default is ''
+        defvalue(str): The replacement string or function for unexisting keys when formating.
 
     Returns:
         str: formatted input
 
     Examples:
-        >>> input = [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}]
+        >>> input = [{'key':'a','val':1},{'key':'b','val':2},{'key':'c'}]
         >>> input | formatdicts()
-        'a : 1\nb : 2\nc : 3\n'
-        >>> input | formatdicts('{key} -> {val}\n')
-        'a -> 1\nb -> 2\nc -> 3\n'
+        'a : 1\nb : 2\nc : -\n'
+        >>> input | formatdicts('{key} -> {val}\n',defvalue='N/A')
+        'a -> 1\nb -> 2\nc -> N/A\n'
         >>> input = [{'name':'Eric','age':47,'level':'guru'},
         ... {'name':'Guido','age':59,'level':'god'}]
         >>> print input | formatdicts('{name}({age}) : {level}\n')   #doctest: +NORMALIZE_WHITESPACE
         Eric(47) : guru
         Guido(59) : god
-        >>> print input | formatdicts('{name}',', ')
+        >>> print input | formatdicts('{name}', ', ')
         Eric, Guido
     """
     @classmethod
-    def op(cls,items,format_str='{key} : {val}\n',join_str = '',*args,**kwargs):
-        return join_str.join([format_str.format(**dct) for dct in items ])
+    def op(cls,items,format_str='{key} : {val}\n',join_str = '',defvalue='-',*args,**kwargs):
+        return join_str.join([dformat(format_str,dct,defvalue) for dct in items ])
 
 class first(TextOp):
     r"""Return the first line/item from the input text
@@ -612,6 +646,8 @@ class head(TextOp):
         b
         >>> ['a','b','c'] | head(2).tolist()
         ['a', 'b']
+        >>> ['a','b','c'] >> head(2)
+        ['a', 'b']
         >>> [('a',1),('b',2),('c',3)] | head(2).tolist()
         [('a', 1), ('b', 2)]
         >>> [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}] | head(2).tolist()
@@ -624,6 +660,39 @@ class head(TextOp):
                 break
             yield line
 
+class skip(TextOp):
+    r"""Skip n lines
+
+    It will return the input text except the n first lines
+
+    Args:
+        lines(int): The number of lines/items to skip.
+
+    Yields:
+        str, lists or dicts: skip 'lines' lines from the input text
+
+    Examples:
+        >>> 'a\nb\nc' | skip(1).tostr()
+        'b\nc'
+        >>> for l in 'a\nb\nc' | skip(1):
+        ...   print l
+        b
+        c
+        >>> ['a','b','c'] | skip(1).tolist()
+        ['b', 'c']
+        >>> ['a','b','c'] >> skip(1)
+        ['b', 'c']
+        >>> [('a',1),('b',2),('c',3)] | skip(1).tolist()
+        [('b', 2), ('c', 3)]
+        >>> [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}] | skip(1).tolist()
+        [{'val': 2, 'key': 'b'}, {'val': 3, 'key': 'c'}]
+    """
+    @classmethod
+    def op(cls,text,lines,*args,**kwargs):
+        for i,line in enumerate(cls._tolist(text)):
+            if i >= lines:
+                yield line
+
 class tail(TextOp):
     r"""Return last lines from the input text
 
@@ -642,6 +711,8 @@ class tail(TextOp):
         c
         >>> ['a','b','c'] | tail(2).tolist()
         ['b', 'c']
+        >>> ['a','b','c'] >> tail(2)
+        ['b', 'c']
         >>> [('a',1),('b',2),('c',3)] | tail(2).tolist()
         [('b', 2), ('c', 3)]
         >>> [{'key':'a','val':1},{'key':'b','val':2},{'key':'c','val':3}] | tail(2).tolist()
@@ -660,7 +731,7 @@ class tail(TextOp):
 class sed(TextOp):
     r"""Replace pattern on-the-fly
 
-    Works like the shell command 'sed'. It uses re.sub() to replace the pattern, this means that
+    Works like the shell command 'sed'. It uses :func:`re.sub` to replace the pattern, this means that
     you can include back-reference into the replacement string.
 
     Args:
@@ -675,6 +746,8 @@ class sed(TextOp):
         'Bonjour Eric\nBonjour Guido'
         >>> [ 'Hello Eric','Hello Guido'] | sed('Hello','Bonjour').tolist()
         ['Bonjour Eric', 'Bonjour Guido']
+        >>> [ 'Hello Eric','Hello Guido'] >> sed('Hello','Bonjour')
+        ['Bonjour Eric', 'Bonjour Guido']
         >>> [ 'Hello Eric','Hello Guido'] | sed(r'$',' !').tolist()
         ['Hello Eric !', 'Hello Guido !']
         >>> import re
@@ -694,7 +767,7 @@ class sed(TextOp):
 class sedi(sed):
     r"""Replace pattern on-the-fly (case insensitive)
 
-    Works like textops.sed_ except that the string as the search pattern is case insensitive.
+    Works like :class:`textops.sed` except that the string as the search pattern is case insensitive.
 
     Args:
         pat(str): a string (case insensitive) or a regular expression for the pattern to search
@@ -706,6 +779,8 @@ class sedi(sed):
     Examples:
         >>> [ 'Hello Eric','Hello Guido'] | sedi('hello','Good bye').tolist()
         ['Good bye Eric', 'Good bye Guido']
+        >>> [ 'Hello Eric','Hello Guido'] >> sedi('hello','Good bye')
+        ['Good bye Eric', 'Good bye Guido']
     """
     flags = re.IGNORECASE
 
@@ -737,6 +812,8 @@ class between(TextOp):
         'b\nc\nd\ne'
         >>> ['a','b','c','d','e','f'] | between('b','e').tolist()
         ['c', 'd']
+        >>> ['a','b','c','d','e','f'] >> between('b','e')
+        ['c', 'd']
         >>> ['a','b','c','d','e','f'] | between('b','e',True,True).tolist()
         ['b', 'c', 'd', 'e']
         >>> input_text = [('a',1),('b',2),('c',3),('d',4),('e',5),('f',6)]
@@ -824,7 +901,7 @@ class between(TextOp):
 class betweeni(between):
     r"""Extract lines between two patterns (case insensitive)
 
-    Works like textops.between_ except patterns are case insensitive
+    Works like :class:`textops.between` except patterns are case insensitive
 
     Args:
         begin(str or regex or list): the pattern(s) to reach before yielding lines from the input
@@ -841,13 +918,15 @@ class betweeni(between):
         []
         >>> ['a','b','c','d','e','f'] | betweeni('B','E').tolist()
         ['c', 'd']
+        >>> ['a','b','c','d','e','f'] >> betweeni('B','E')
+        ['c', 'd']
     """
     flags = re.IGNORECASE
 
 class betweenb(between):
     r"""Extract lines between two patterns (includes boundaries)
 
-    Works like textops.between_ except it return boundaries by default that is
+    Works like :class:`textops.between` except it return boundaries by default that is
     get_begin = get_end = True.
 
     Args:
@@ -863,13 +942,15 @@ class betweenb(between):
     Examples:
         >>> ['a','b','c','d','e','f'] | betweenb('b','e').tolist()
         ['b', 'c', 'd', 'e']
+        >>> ['a','b','c','d','e','f'] >> betweenb('b','e')
+        ['b', 'c', 'd', 'e']
     """
     boundaries = True
 
 class betweenbi(betweenb):
     r"""Extract lines between two patterns (includes boundaries and case insensitive)
 
-    Works like textops.between_ except patterns are case insensitive and it yields boundaries too.
+    Works like :class:`textops.between` except patterns are case insensitive and it yields boundaries too.
     That is get_begin = get_end = True.
 
     Args:
@@ -887,6 +968,8 @@ class betweenbi(betweenb):
         []
         >>> ['a','b','c','d','e','f'] | betweenbi('B','E').tolist()
         ['b', 'c', 'd', 'e']
+        >>> ['a','b','c','d','e','f'] >> betweenbi('B','E')
+        ['b', 'c', 'd', 'e']
     """
     flags = re.IGNORECASE
 
@@ -896,13 +979,18 @@ class linetester(TextOp):
     def testline(cls, to_test, *args,**kwargs):
         raise AssertionError('Method testline must be defined in derivated class.')
 
+    @classmethod
+    def cast_to(cls, *args, **kwargs):
+        return type(args[0])
+
     @classmethod
     def op(cls, text, *args,**kwargs):
         col_or_key = kwargs.get('col_or_key')
+        cast_to = cls.cast_to(*args,**kwargs)
         for line in cls._tolist(text):
             try:
                 to_test = line if col_or_key is None else line[col_or_key]
-                to_test = to_test if isinstance(to_test, basestring) else str(to_test)
+                to_test = cast_to(to_test)
                 if cls.testline(to_test, *args,**kwargs):
                     yield line
             except (ValueError, TypeError, IndexError, KeyError):
@@ -918,9 +1006,15 @@ class inrange(linetester):
     ``inrange`` works for any kind of list of strings, but also for list of lists and list of dicts.
     In these cases, one can test only one column or one key but return the whole list/dict.
 
+    Each strings that will be tested is converted with the same type of the first argument.
+
     Args:
         begin(str): range begin string
         end(str): range end string
+        get_begin(bool): if True : include lines having the same value as the range begin,
+            Default : True
+        get_end(bool): if True : include lines having the same value as the range end,
+            Default : False
         col_or_key (int or str): test only one column or one key (optional).
             it *MUST BE PASSED BY NAME* if you want to use this argument.
 
@@ -934,6 +1028,8 @@ class inrange(linetester):
         ... 2015-11-05 ddd'''
         >>> logs | inrange('2015-08-12','2015-11-05').tolist()
         ['2015-08-23 bbbb', '2015-09-14 ccc']
+        >>> logs >> inrange('2015-08-12','2015-11-05')
+        ['2015-08-23 bbbb', '2015-09-14 ccc']
         >>> logs = [ ('aaaa','2015-08-11'),
         ... ('bbbb','2015-08-23'),
         ... ('ccc','2015-09-14'),
@@ -946,19 +1042,33 @@ class inrange(linetester):
         ... {'data':'ddd','date':'2015-11-05'} ]
         >>> logs | inrange('2015-08-12','2015-11-05',col_or_key='date').tolist()
         [{'date': '2015-08-23', 'data': 'bbbb'}, {'date': '2015-09-14', 'data': 'ccc'}]
+        >>> ints = '1\n2\n01\n02\n11\n12\n22\n20'
+        >>> ints | inrange(1,3).tolist()
+        ['1', '2', '01', '02']
+        >>> ints | inrange('1','3').tolist()
+        ['1', '2', '11', '12', '22', '20']
+        >>> ints | inrange('1','3',get_begin=False).tolist()
+        ['2', '11', '12', '22', '20']
     """
     @classmethod
-    def testline(cls, to_test,  begin, end, *args,**kwargs):
-        return to_test >= begin and to_test < end
+    def testline(cls, to_test,  begin, end, get_begin=True, get_end=False, *args,**kwargs):
+        return  (to_test > begin and to_test < end) or \
+                (to_test == begin and get_begin) or \
+                (to_test == end and get_end)
+
 
 class outrange(linetester):
     r"""Extract lines NOT between a range of strings
 
-    Works like textops.inrange_ except it yields lines that are NOT in the range
+    Works like :class:`textops.inrange` except it yields lines that are NOT in the range
 
     Args:
         begin(str): range begin string
         end(str): range end string
+        get_begin(bool): if True : include lines having the same value as the range begin,
+            Default : False
+        get_end(bool): if True : include lines having the same value as the range end,
+            Default : False
         col_or_key (int or str): test only one column or one key (optional).
             it *MUST BE PASSED BY NAME* if you want to use this argument.
 
@@ -972,16 +1082,151 @@ class outrange(linetester):
         ... 2015-11-05 ddd'''
         >>> logs | outrange('2015-08-12','2015-11-05').tolist()
         ['2015-08-11 aaaa', '2015-11-05 ddd']
+        >>> logs | outrange('2015-08-23 bbbb','2015-09-14 ccc').tolist()
+        ['2015-08-11 aaaa', '2015-11-05 ddd']
+        >>> logs | outrange('2015-08-23 bbbb','2015-09-14 ccc', get_begin=True).tolist()
+        ['2015-08-11 aaaa', '2015-08-23 bbbb', '2015-11-05 ddd']
     """
     @classmethod
-    def testline(cls, to_test,  begin, end, *args,**kwargs):
-        return not (to_test >= begin and to_test < end)
+    def testline(cls, to_test,  begin, end, get_begin=False, get_end=False, *args,**kwargs):
+        return       to_test < begin or to_test > end or \
+                     (to_test == begin and get_begin) or \
+                     (to_test == end and get_end)
+
+class lessthan(linetester):
+    r"""Extract lines with value strictly less than specified string
+
+    It works for any kind of list of strings, but also for list of lists and list of dicts.
+    In these cases, one can test only one column or one key but return the whole list/dict.
+
+    Each strings that will be tested is temporarily converted with the same type as the first
+    argument given to ``lessthan`` (see examples).
+
+    Args:
+        value(str): string to test with
+        col_or_key (int or str): test only one column or one key (optional).
+            it *MUST BE PASSED BY NAME* if you want to use this argument.
+
+    Yields:
+        str or list or dict: lines having values strictly less than the specified reference value
+
+    Examples:
+        >>> logs = '''2015-08-11 aaaa
+        ... 2015-08-23 bbbb
+        ... 2015-09-14 ccc
+        ... 2015-11-05 ddd'''
+        >>> logs | lessthan('2015-09-14').tolist()
+        ['2015-08-11 aaaa', '2015-08-23 bbbb']
+        >>> logs = [ ('aaaa','2015-08-11'),
+        ... ('bbbb','2015-08-23'),
+        ... ('ccc','2015-09-14'),
+        ... ('ddd','2015-11-05') ]
+        >>> logs | lessthan('2015-11-05',col_or_key=1).tolist()
+        [('aaaa', '2015-08-11'), ('bbbb', '2015-08-23'), ('ccc', '2015-09-14')]
+        >>> logs = [ {'data':'aaaa','date':'2015-08-11'},
+        ... {'data':'bbbb','date':'2015-08-23'},
+        ... {'data':'ccc','date':'2015-09-14'},
+        ... {'data':'ddd','date':'2015-11-05'} ]
+        >>> logs | lessthan('2015-09-14',col_or_key='date').tolist()
+        [{'date': '2015-08-11', 'data': 'aaaa'}, {'date': '2015-08-23', 'data': 'bbbb'}]
+        >>> ints = '1\n2\n01\n02\n11\n12\n22\n20'
+        >>> ints | lessthan(3).tolist()
+        ['1', '2', '01', '02']
+        >>> ints | lessthan('3').tolist()
+        ['1', '2', '01', '02', '11', '12', '22', '20']
+        """
+    @classmethod
+    def testline(cls, to_test, value, *args,**kwargs):
+        return to_test < value
+
+class lessequal(linetester):
+    r"""Extract lines with value strictly less than specified string
+
+    It works like :class:`textops.lessthan` except the test is "less or equal"
+
+    Args:
+        value(str): string to test with
+        col_or_key (int or str): test only one column or one key (optional).
+            it *MUST BE PASSED BY NAME* if you want to use this argument.
+
+    Yields:
+        str or list or dict: lines having values less than or equal to the specified value
+
+    Examples:
+        >>> logs = '''2015-08-11 aaaa
+        ... 2015-08-23 bbbb
+        ... 2015-09-14 ccc
+        ... 2015-11-05 ddd'''
+        >>> logs | lessequal('2015-09-14').tolist()
+        ['2015-08-11 aaaa', '2015-08-23 bbbb']
+        >>> logs >> lessequal('2015-09-14')
+        ['2015-08-11 aaaa', '2015-08-23 bbbb']
+        >>> logs | lessequal('2015-09-14 ccc').tolist()
+        ['2015-08-11 aaaa', '2015-08-23 bbbb', '2015-09-14 ccc']
+        """
+    @classmethod
+    def testline(cls, to_test, value, *args,**kwargs):
+        return to_test <= value
+
+class greaterthan(linetester):
+    r"""Extract lines with value strictly less than specified string
+
+    It works like :class:`textops.lessthan` except the test is "greater than"
+
+    Args:
+        value(str): string to test with
+        col_or_key (int or str): test only one column or one key (optional).
+            it *MUST BE PASSED BY NAME* if you want to use this argument.
+
+    Yields:
+        str or list or dict: lines having values greater than the specified value
+
+    Examples:
+        >>> logs = '''2015-08-11 aaaa
+        ... 2015-08-23 bbbb
+        ... 2015-09-14 ccc
+        ... 2015-11-05 ddd'''
+        >>> logs | greaterthan('2015-09-14 ccc').tolist()
+        ['2015-11-05 ddd']
+        >>> logs >> greaterthan('2015-09-14 ccc')
+        ['2015-11-05 ddd']
+        """
+    @classmethod
+    def testline(cls, to_test, value, *args,**kwargs):
+        return to_test > value
+
+class greaterequal(linetester):
+    r"""Extract lines with value strictly less than specified string
+
+    It works like :class:`textops.greaterthan` except the test is "greater than or equal to"
+
+    Args:
+        value(str): string to test with
+        col_or_key (int or str): test only one column or one key (optional).
+            it *MUST BE PASSED BY NAME* if you want to use this argument.
+
+    Yields:
+        str or list or dict: lines having values greater than or equal to the specified value
+
+    Examples:
+        >>> logs = '''2015-08-11 aaaa
+        ... 2015-08-23 bbbb
+        ... 2015-09-14 ccc
+        ... 2015-11-05 ddd'''
+        >>> logs | greaterequal('2015-09-14 ccc').tolist()
+        ['2015-09-14 ccc', '2015-11-05 ddd']
+        >>> logs >> greaterequal('2015-09-14 ccc')
+        ['2015-09-14 ccc', '2015-11-05 ddd']
+        """
+    @classmethod
+    def testline(cls, to_test, value, *args,**kwargs):
+        return to_test >= value
 
 class before(between):
     r"""Extract lines before a patterns
 
-    Works like textops.between_ except that it requires only the ending pattern : it will yields
-    all line from the input text beginning until the specified pattern has been reached.
+    Works like :class:`textops.between` except that it requires only the ending pattern : it will yields
+    all lines from the input text beginning until the specified pattern has been reached.
 
     Args:
         pattern(str or regex or list): no more lines are yield after reaching this pattern(s)
@@ -989,7 +1234,7 @@ class before(between):
         col_or_key (int or str): test only one column or one key (optional)
 
     Yields:
-        str or list or dict: lines between two patterns
+        str or list or dict: lines before the specified pattern
 
     Examples:
         >>> ['a','b','c','d','e','f'] | before('c').tolist()
@@ -999,6 +1244,8 @@ class before(between):
         >>> input_text = [{'k':1},{'k':2},{'k':3},{'k':4},{'k':5},{'k':6}]
         >>> input_text | before('3',col_or_key='k').tolist()
         [{'k': 1}, {'k': 2}]
+        >>> input_text >> before('3',col_or_key='k')
+        [{'k': 1}, {'k': 2}]
     """
     @classmethod
     def op(cls, text, pattern, get_end=False, col_or_key=None,*args,**kwargs):
@@ -1007,15 +1254,15 @@ class before(between):
 class beforei(before):
     r"""Extract lines before a patterns (case insensitive)
 
-    Works like textops.before_ except that the pattern is case insensitive.
+    Works like :class:`textops.before` except that the pattern is case insensitive.
 
     Args:
         pattern(str or regex or list): no more lines are yield after reaching this pattern(s)
-        get_end(bool): if True : include the line matching the end pattern (Default : False)
+        get_end(bool): if True : include the line matching the pattern (Default : False)
         col_or_key (int or str): test only one column or one key (optional)
 
     Yields:
-        str or list or dict: lines between two patterns
+        str or list or dict: lines before the specified pattern
 
     Examples:
         >>> ['a','b','c','d','e','f'] | before('C').tolist()
@@ -1024,23 +1271,106 @@ class beforei(before):
         ['a', 'b']
         >>> ['a','b','c','d','e','f'] | beforei('C',True).tolist()
         ['a', 'b', 'c']
+        >>> ['a','b','c','d','e','f'] >> beforei('C',True)
+        ['a', 'b', 'c']
     """
     flags = re.IGNORECASE
 
 class after(between):
+    r"""Extract lines after a patterns
+
+    Works like :class:`textops.before` except that it will yields all lines from the input AFTER the given
+    pattern has been found.
+
+    Args:
+        pattern(str or regex or list): start yielding lines after reaching this pattern(s)
+        get_begin(bool): if True : include the line matching the pattern (Default : False)
+        col_or_key (int or str): test only one column or one key (optional)
+
+    Yields:
+        str or list or dict: lines after the specified pattern
+
+    Examples:
+        >>> ['a','b','c','d','e','f'] | after('c').tolist()
+        ['d', 'e', 'f']
+        >>> ['a','b','c','d','e','f'] | after('c',True).tolist()
+        ['c', 'd', 'e', 'f']
+        >>> input_text = [{'k':1},{'k':2},{'k':3},{'k':4},{'k':5},{'k':6}]
+        >>> input_text | after('3',col_or_key='k').tolist()
+        [{'k': 4}, {'k': 5}, {'k': 6}]
+        >>> input_text >> after('3',col_or_key='k')
+        [{'k': 4}, {'k': 5}, {'k': 6}]
+    """
     @classmethod
     def op(cls, text, pattern, get_begin=False,*args,**kwargs):
-        return between.op(text,pattern,None,get_begin=get_begin)
+        return super(after,cls).op(text,pattern,None,get_begin=get_begin)
 
-class afteri(after): flags = re.IGNORECASE
+class afteri(after):
+    r"""Extract lines after a patterns (case insensitive)
+
+    Works like :class:`textops.after` except that the pattern is case insensitive.
+
+    Args:
+        pattern(str or regex or list): no more lines are yield after reaching this pattern(s)
+        get_begin(bool): if True : include the line matching the pattern (Default : False)
+        col_or_key (int or str): test only one column or one key (optional)
+
+    Yields:
+        str or list or dict: lines before the specified pattern
+
+    Examples:
+        >>> ['a','b','c','d','e','f'] | after('C').tolist()
+        []
+        >>> ['a','b','c','d','e','f'] | afteri('C').tolist()
+        ['d', 'e', 'f']
+        >>> ['a','b','c','d','e','f'] | afteri('C',True).tolist()
+        ['c', 'd', 'e', 'f']
+        >>> ['a','b','c','d','e','f'] >> afteri('C',True)
+        ['c', 'd', 'e', 'f']
+    """
+    flags = re.IGNORECASE
 
 class mapfn(TextOp):
+    r"""Apply a specified function on every line
+
+    It works like the python map() function.
+
+    Args:
+        map_fn(callable): a function or a callable to apply on every line
+
+    Yields:
+        any: lines modified by the map_fn function
+
+    Examples:
+        >>> ['a','b','c'] | mapfn(lambda l:l*2).tolist()
+        ['aa', 'bb', 'cc']
+        >>> ['a','b','c'] >> mapfn(lambda l:l*2)
+        ['aa', 'bb', 'cc']
+    """
     @classmethod
     def op(cls, text, map_fn, *args,**kwargs):
         for line in cls._tolist(text):
             yield map_fn(line)
 
 class iffn(TextOp):
+    r"""Filters the input text with a specified function
+
+    It works like the python filter() fonction.
+
+    Args:
+        filter_fn(callable): a function to be called against each line and returning a boolean.
+        True means : yield the line.
+
+    Yields:
+        any: lines filtered by the filter_fn function
+
+    Examples:
+        >>> import re
+        >>> 'line1\nline2\nline3\nline4' | iffn(lambda l:int(re.sub(r'\D','',l)) % 2).tolist()
+        ['line1', 'line3']
+        >>> 'line1\nline2\nline3\nline4' >> iffn(lambda l:int(re.sub(r'\D','',l)) % 2)
+        ['line1', 'line3']
+    """
     @classmethod
     def op(cls, text, filter_fn=None, *args,**kwargs):
         if filter_fn is None:
@@ -1050,6 +1380,25 @@ class iffn(TextOp):
                 yield line
 
 class mapif(TextOp):
+    r"""Filters and maps the input text with 2 specified functions
+
+    Filters input text AND apply a map function on every filtered lines.
+
+    Args:
+        map_fn(callable): a function or a callable to apply on every line to be yield
+        filter_fn(callable): a function to be called against each line and returning a boolean.
+        True means : yield the line.
+
+    Yields:
+        any: lines filtered by the filter_fn function and modified by map_fn function
+
+    Examples:
+        >>> import re
+        >>> 'a1\nb2\nc3\nd4' | mapif(lambda l:l*2,lambda l:int(re.sub(r'\D','',l)) % 2).tolist()
+        ['a1a1', 'c3c3']
+        >>> 'a1\nb2\nc3\nd4' >> mapif(lambda l:l*2,lambda l:int(re.sub(r'\D','',l)) % 2)
+        ['a1a1', 'c3c3']
+    """
     @classmethod
     def op(cls, text, map_fn, filter_fn=None,*args,**kwargs):
         if filter_fn is None:
@@ -1059,11 +1408,44 @@ class mapif(TextOp):
                 yield map_fn(line)
 
 class doreduce(TextOp):
+    r"""Reduce the input text
+
+    Uses python reduce() function.
+
+    Args:
+        reduce_fn(callable): a function or a callable to reduce every line.
+        initializer(object): initial accumulative value (Default : None)
+
+    Returns:
+        any: reduced value
+
+    Examples:
+        >>> import re
+        >>> 'a1\nb2\nc3\nd4' | doreduce(lambda x,y:x+re.sub(r'\d','',y),'')
+        'abcd'
+        >>> 'a1\nb2\nc3\nd4' >> doreduce(lambda x,y:x+re.sub(r'\d','',y),'')
+        'abcd'
+    """
     @classmethod
-    def op(cls, text, reduce_fn, *args, **kwargs):
-        return reduce(reduce_fn, cls._tolist(text))
+    def op(cls, text, reduce_fn, initializer=None, *args, **kwargs):
+        return reduce(reduce_fn, cls._tolist(text), initializer)
 
 class merge_dicts(TextOp):
+    r"""Merge a list of dicts into one single dict
+
+    Returns:
+        dict: merged dicts
+
+    Examples:
+        >>> pattern=r'item="(?P<item>[^"]*)" count="(?P<i_count>[^"]*)" price="(?P<i_price>[^"]*)"'
+        >>> s='item="col1" count="col2" price="col3"\nitem="col11" count="col22" price="col33"'
+        >>> s | cutkv(pattern,key_name='item')                                                      # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+        [{'col1': {'item': 'col1', 'i_price': 'col3', 'i_count': 'col2'}},...
+        {'col11': {'item': 'col11', 'i_price': 'col33', 'i_count': 'col22'}}]
+        >>> s | cutkv(pattern,key_name='item').merge_dicts()                                        # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+        {'col11': {'item': 'col11', 'i_price': 'col33', 'i_count': 'col22'},...
+        'col1': {'item': 'col1', 'i_price': 'col3', 'i_count': 'col2'}}
+    """
     @classmethod
     def op(cls,text,*args,**kwargs):
         out = {}
@@ -1073,25 +1455,140 @@ class merge_dicts(TextOp):
         return out
 
 class span(TextOp):
+    r"""Ensure that a list of lists has exactly the specified number of column
+
+    This is useful in for-loop with multiple assignment
+
+    Args:
+        nbcols(int): number columns to return
+        fill_str(str): the value to return for not exsiting columns
+
+    Returns:
+        list: A list with exactly ``nbcols`` columns
+
+    Examples:
+        >>> s='a\nb c\nd e f g h\ni j k\n\n'
+        >>> s | cut()
+        [['a'], ['b', 'c'], ['d', 'e', 'f', 'g', 'h'], ['i', 'j', 'k'], []]
+        >>> s | cut().span(3,'-').tolist()
+        [['a', '-', '-'], ['b', 'c', '-'], ['d', 'e', 'f'], ['i', 'j', 'k'], ['-', '-', '-']]
+        >>> s >> cut().span(3,'-')
+        [['a', '-', '-'], ['b', 'c', '-'], ['d', 'e', 'f'], ['i', 'j', 'k'], ['-', '-', '-']]
+        >>> for x,y,z in s | cut().span(3,'-'):
+        ...    print x,y,z
+        a - -
+        b c -
+        d e f
+        i j k
+        - - -
+    """
     @classmethod
     def op(cls, text, nbcols, fill_str='', *args,**kwargs):
         fill_list = [fill_str] * nbcols
         for sublist in cls._tolist(text):
             yield (sublist+fill_list)[:nbcols]
 
-class slice(TextOp):
+class doslice(TextOp):
+    r"""Get lines/items from ``begin`` line to ``end`` line with some ``step``
+
+    Args:
+        begin(int): first line number to get.
+            must be None or an integer: 0 <= x <= maxint
+        end(int): end line number (get lines up to end - 1).
+            must be None or an integer: 0 <= x <= maxint
+        step(int): get every ``step`` line (Default : 1)
+
+    Returns:
+        generator: A slice of the original text
+
+    Examples:
+        >>> s='a\nb\nc\nd\ne\nf'
+        >>> s | doslice(1,4).tolist()
+        ['b', 'c', 'd']
+        >>> s >> doslice(1,4)
+        ['b', 'c', 'd']
+        >>> s >> doslice(2)
+        ['c', 'd', 'e', 'f']
+        >>> s >> doslice(0,4,2)
+        ['a', 'c']
+        >>> s >> doslice(None,None,2)
+        ['a', 'c', 'e']
+    """
     @classmethod
-    def op(cls, text, begin=0, end=sys.maxsize, step = 1, fill_str=None, *args,**kwargs):
+    def op(cls, text, begin=0, end=sys.maxsize, step = 1, *args,**kwargs):
+        for line in itertools.islice(cls._tolist(text), begin, end, step):
+            yield line
+
+class subslice(TextOp):
+    r"""Get a slice of columns for list of lists
+
+    Args:
+        begin(int): first columns number to get.
+            must be None or an integer: 0 <= x <= maxint
+        end(int): end columns number (get columns up to end - 1).
+            must be None or an integer: 0 <= x <= maxint
+        step(int): get every ``step`` columns (Default : 1)
+
+    Returns:
+        generator: A slice of the original text
+
+    Examples:
+        >>> s='a\nb c\nd e f g h\ni j k\n\n'
+        >>> s | cut().span(3,'-').tolist()
+        [['a', '-', '-'], ['b', 'c', '-'], ['d', 'e', 'f'], ['i', 'j', 'k'], ['-', '-', '-']]
+        >>> s | cut().span(3,'-').subslice(1,3).tolist()
+        [['-', '-'], ['c', '-'], ['e', 'f'], ['j', 'k'], ['-', '-']]
+        >>> s >> cut().span(3,'-').subslice(1,3)
+        [['-', '-'], ['c', '-'], ['e', 'f'], ['j', 'k'], ['-', '-']]
+    """
+    @classmethod
+    def op(cls, text, begin=0, end=sys.maxsize, step = 1, *args,**kwargs):
         for sublist in cls._tolist(text):
             yield sublist[begin:end:step]
 
 class subitem(TextOp):
+    r"""Get a specified column for list of lists
+
+    Args:
+        n(int): column number to get.
+
+    Returns:
+        generator: A list
+
+    Examples:
+        >>> s='a\nb c\nd e f g h\ni j k\n\n'
+        >>> s | cut().span(3,'-').tolist()
+        [['a', '-', '-'], ['b', 'c', '-'], ['d', 'e', 'f'], ['i', 'j', 'k'], ['-', '-', '-']]
+        >>> s | cut().span(3,'-').subitem(1).tolist()
+        ['-', 'c', 'e', 'j', '-']
+        >>> s >> cut().span(3,'-').subitem(1)
+        ['-', 'c', 'e', 'j', '-']
+        >>> s >> cut().span(3,'-').subitem(-1)
+        ['-', '-', 'f', 'k', '-']
+    """
     @classmethod
     def op(cls, text, n, *args,**kwargs):
         for sublist in cls._tolist(text):
             yield sublist[n]
 
 class subitems(TextOp):
+    r"""Get the specified columns for list of lists
+
+    Args:
+        ntab(list of int): columns numbers to get.
+
+    Returns:
+        generator: A list of lists
+
+    Examples:
+        >>> s='a\nb c\nd e f g h\ni j k\n\n'
+        >>> s | cut().span(3,'-').tolist()
+        [['a', '-', '-'], ['b', 'c', '-'], ['d', 'e', 'f'], ['i', 'j', 'k'], ['-', '-', '-']]
+        >>> s | cut().span(3,'-').subitems([0,2]).tolist()
+        [['a', '-'], ['b', '-'], ['d', 'f'], ['i', 'k'], ['-', '-']]
+        >>> s >> cut().span(3,'-').subitems('0,2')
+        [['a', '-'], ['b', '-'], ['d', 'f'], ['i', 'k'], ['-', '-']]
+    """
     @classmethod
     def op(cls, text, ntab, *args,**kwargs):
         if isinstance(ntab,basestring):
@@ -1100,6 +1597,32 @@ class subitems(TextOp):
             yield [ sublist[n] for n in ntab ]
 
 class uniq(TextOp):
+    r"""Remove all line repetitions
+
+    If a line is many times in the same text (even if there are some different lines between),
+    only the first will be taken. Works also with list of lists or dicts.
+
+    Returns:
+        generator: Unified text line by line.
+
+    Examples:
+        >>> s='f\na\nb\na\nc\nc\ne\na\nc\nf'
+        >>> s >> uniq()
+        ['f', 'a', 'b', 'c', 'e']
+        >>> for line in s | uniq():
+        ...     print line
+        f
+        a
+        b
+        c
+        e
+        >>> l = [ [1,2], [3,4], [1,2] ]
+        >>> l >> uniq()
+        [[1, 2], [3, 4]]
+        >>> d = [ {'a':1}, {'b':2}, {'a':1} ]
+        >>> d >> uniq()
+        [{'a': 1}, {'b': 2}]
+    """
     @classmethod
     def op(cls, text, *args,**kwargs):
         s=[]
